@@ -627,12 +627,13 @@ type CacheGCSConfig struct {
 
 //nolint:lll
 type CacheS3Config struct {
-	ServerAddress  string `toml:"ServerAddress,omitempty" long:"server-address" env:"CACHE_S3_SERVER_ADDRESS" description:"A host:port to the used S3-compatible server"`
-	AccessKey      string `toml:"AccessKey,omitempty" long:"access-key" env:"CACHE_S3_ACCESS_KEY" description:"S3 Access Key"`
-	SecretKey      string `toml:"SecretKey,omitempty" long:"secret-key" env:"CACHE_S3_SECRET_KEY" description:"S3 Secret Key"`
-	BucketName     string `toml:"BucketName,omitempty" long:"bucket-name" env:"CACHE_S3_BUCKET_NAME" description:"Name of the bucket where cache will be stored"`
-	BucketLocation string `toml:"BucketLocation,omitempty" long:"bucket-location" env:"CACHE_S3_BUCKET_LOCATION" description:"Name of S3 region"`
-	Insecure       bool   `toml:"Insecure,omitempty" long:"insecure" env:"CACHE_S3_INSECURE" description:"Use insecure mode (without https)"`
+	ServerAddress      string     `toml:"ServerAddress,omitempty" long:"server-address" env:"CACHE_S3_SERVER_ADDRESS" description:"A host:port to the used S3-compatible server"`
+	AccessKey          string     `toml:"AccessKey,omitempty" long:"access-key" env:"CACHE_S3_ACCESS_KEY" description:"S3 Access Key"`
+	SecretKey          string     `toml:"SecretKey,omitempty" long:"secret-key" env:"CACHE_S3_SECRET_KEY" description:"S3 Secret Key"`
+	BucketName         string     `toml:"BucketName,omitempty" long:"bucket-name" env:"CACHE_S3_BUCKET_NAME" description:"Name of the bucket where cache will be stored"`
+	BucketLocation     string     `toml:"BucketLocation,omitempty" long:"bucket-location" env:"CACHE_S3_BUCKET_LOCATION" description:"Name of S3 region"`
+	Insecure           bool       `toml:"Insecure,omitempty" long:"insecure" env:"CACHE_S3_INSECURE" description:"Use insecure mode (without https)"`
+	AuthenticationType S3AuthType `toml:"AuthenticationType,omitempty" long:"authentication_type" env:"CACHE_S3_AUTHENTICATION_TYPE" description:"IAM or credentials"`
 }
 
 //nolint:lll
@@ -736,8 +737,30 @@ type CustomBuildDir struct {
 	Enabled bool `toml:"enabled,omitempty" json:"enabled" long:"enabled" env:"CUSTOM_BUILD_DIR_ENABLED" description:"Enable job specific build directories"`
 }
 
-func (c *CacheS3Config) ShouldUseIAMCredentials() bool {
-	return c.ServerAddress == "" || c.AccessKey == "" || c.SecretKey == ""
+type S3AuthType string
+
+const (
+	S3AuthTypeAccessKey S3AuthType = "access-key"
+	S3AuthTypeIAM       S3AuthType = "iam"
+)
+
+func (c *CacheS3Config) AuthType() S3AuthType {
+	switch c.AuthenticationType {
+	case S3AuthTypeAccessKey, S3AuthTypeIAM:
+		return c.AuthenticationType
+	}
+
+	if c.AuthenticationType != "" {
+		return ""
+	}
+
+	// TODO: deprecate this path in 15.0. Users will need to explicitly define AuthenticationType
+	// https://gitlab.com/gitlab-org/gitlab-runner/-/issues/28171
+	if c.ServerAddress == "" || c.AccessKey == "" || c.SecretKey == "" {
+		return S3AuthTypeIAM
+	}
+
+	return S3AuthTypeAccessKey
 }
 
 func (c *CacheConfig) GetPath() string {
