@@ -20,7 +20,6 @@ import (
 	"github.com/docker/docker/api/types/container"
 	"github.com/docker/docker/api/types/network"
 	"github.com/docker/docker/pkg/stdcopy"
-	"github.com/kardianos/osext"
 	"github.com/sirupsen/logrus"
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
@@ -97,7 +96,7 @@ type executor struct {
 }
 
 func init() {
-	runnerFolder, err := osext.ExecutableFolder()
+	runner, err := os.Executable()
 	if err != nil {
 		logrus.Errorln(
 			"Docker executor: unable to detect gitlab-runner folder, "+
@@ -106,6 +105,8 @@ func init() {
 		)
 	}
 
+	runnerFolder := filepath.Dir(runner)
+
 	PrebuiltImagesPaths = []string{
 		// When gitlab-runner is running from repository root
 		filepath.Join(runnerFolder, "out/helper-images"),
@@ -404,6 +405,7 @@ func (e *executor) createHostConfigForService() *container.HostConfig {
 		RestartPolicy: neverRestartPolicy,
 		ExtraHosts:    e.Config.Docker.ExtraHosts,
 		Privileged:    e.Config.Docker.Privileged,
+		Runtime:       e.Config.Docker.Runtime,
 		UsernsMode:    container.UsernsMode(e.Config.Docker.UsernsMode),
 		NetworkMode:   e.networkMode,
 		Binds:         e.volumesManager.Binds(),
@@ -731,7 +733,7 @@ func (e *executor) createHostConfig() (*container.HostConfig, error) {
 }
 
 func (e *executor) startAndWatchContainer(ctx context.Context, id string, input io.Reader) error {
-	dockerExec := exec.NewDocker(e.client, e.waiter, e.Build.Log())
+	dockerExec := exec.NewDocker(e.Context, e.client, e.waiter, e.Build.Log())
 
 	return dockerExec.Exec(ctx, id, input, e.Trace)
 }
