@@ -19,6 +19,7 @@ import (
 	"github.com/gorilla/websocket"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+	v1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/labels"
 	k8s "k8s.io/client-go/kubernetes"
@@ -32,6 +33,7 @@ import (
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
 	"gitlab.com/gitlab-org/gitlab-runner/session"
 	"gitlab.com/gitlab-org/gitlab-runner/shells"
+	"gitlab.com/gitlab-org/gitlab-runner/shells/shellstest"
 )
 
 type featureFlagTest func(t *testing.T, flagName string, flagValue bool)
@@ -61,6 +63,8 @@ func TestRunIntegrationTestsWithFeatureFlag(t *testing.T) {
 		"testConfiguredBuildDirVolumeMountFeatureFlag":            testBuildDirVolumeMountFeatureFlag,
 		"testUserConfiguredBuildDirVolumeMountFeatureFlag":        testUserConfiguredBuildDirVolumeMountFeatureFlag,
 		"testKubernetesPwshFeatureFlag":                           testKubernetesPwshFeatureFlag,
+		"testKubernetesContainerHookFeatureFlag":                  testKubernetesContainerHookFeatureFlag,
+		"testKubernetesGarbageCollection":                         testKubernetesGarbageCollection,
 	}
 
 	featureFlags := []string{
@@ -91,6 +95,22 @@ func testKubernetesSuccessRunFeatureFlag(t *testing.T, featureFlagName string, f
 	assert.NoError(t, err)
 }
 
+func TestBuildScriptSections(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
+
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		if shell == "cmd" || shell == "pwsh" || shell == "powershell" {
+			// support for pwsh and powershell tracked in https://gitlab.com/gitlab-org/gitlab-runner/-/issues/28119
+			t.Skip("CMD, pwsh, powershell not supported")
+		}
+
+		build := getTestBuild(t, common.GetRemoteSuccessfulBuild)
+		build.Runner.RunnerSettings.Shell = shell
+
+		buildtest.RunBuildWithSections(t, build)
+	})
+}
+
 func testKubernetesMultistepRunFeatureFlag(t *testing.T, featureFlagName string, featureFlagValue bool) {
 	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
 
@@ -312,8 +332,7 @@ func testKubernetesMissingImageFeatureFlag(t *testing.T, featureFlagName string,
 
 	err := build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
 	require.Error(t, err)
-	var buildErr *common.BuildError
-	assert.ErrorAs(t, err, &buildErr)
+	assert.ErrorIs(t, err, &common.BuildError{FailureReason: common.ScriptFailure})
 	assert.Contains(t, err.Error(), "image pull failed")
 }
 
@@ -326,8 +345,7 @@ func testKubernetesMissingTagFeatureFlag(t *testing.T, featureFlagName string, f
 
 	err := build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
 	require.Error(t, err)
-	var buildErr *common.BuildError
-	assert.ErrorAs(t, err, &buildErr)
+	assert.ErrorIs(t, err, &common.BuildError{FailureReason: common.ScriptFailure})
 	assert.Contains(t, err.Error(), "image pull failed")
 }
 
@@ -567,6 +585,170 @@ func testUserConfiguredBuildDirVolumeMountFeatureFlag(t *testing.T, featureFlagN
 	assert.NoError(t, err)
 }
 
+// testKubernetesGarbageCollection tests the deletion of resources via garbage collector once the owning pod is deleted
+func testKubernetesGarbageCollection(t *testing.T, featureFlagName string, featureFlagValue bool) {
+	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
+
+	ctxTimeout := time.Minute
+	client := getTestKubeClusterClient(t)
+
+	validateResourcesCreated := func(
+		t *testing.T,
+		client *k8s.Clientset,
+		featureFlagValue bool,
+		namespace string,
+		podName string,
+	) {
+		credentials, err := getSecrets(client, namespace, podName)
+		require.NoError(t, err)
+		configMaps, err := getConfigMaps(client, namespace, podName)
+		require.NoError(t, err)
+
+		assert.NotEmpty(t, credentials)
+		if !featureFlagValue {
+			assert.NotEmpty(t, configMaps)
+		} else {
+			assert.Empty(t, configMaps)
+		}
+	}
+
+	validateResourcesDeleted := func(t *testing.T, client *k8s.Clientset, namespace string, podName string) {
+		credentials, err := getSecrets(client, namespace, podName)
+		require.NoError(t, err)
+		configMaps, err := getConfigMaps(client, namespace, podName)
+		require.NoError(t, err)
+
+		assert.Empty(t, credentials)
+		assert.Empty(t, configMaps)
+	}
+
+	tests := map[string]struct {
+		namespace string
+		init      func(t *testing.T, build *common.Build, client *k8s.Clientset, namespace string)
+		finalize  func(t *testing.T, client *k8s.Clientset, namespace string)
+	}{
+		"pod deletion during build step": {
+			namespace: "default",
+		},
+		"pod deletion during prepare step": {
+			namespace: "default",
+		},
+		"pod deletion during prepare stage in custom namespace": {
+			namespace: "garbage-collection-namespace",
+			init: func(t *testing.T, build *common.Build, client *k8s.Clientset, namespace string) {
+				ctx, cancel := context.WithTimeout(context.Background(), ctxTimeout)
+				defer cancel()
+
+				k8sNamespace := &v1.Namespace{
+					ObjectMeta: metav1.ObjectMeta{
+						Name:         namespace,
+						GenerateName: namespace,
+					},
+				}
+				k8sNamespace, err := client.CoreV1().Namespaces().Create(ctx, k8sNamespace, metav1.CreateOptions{})
+				require.NoError(t, err)
+
+				credentials, err := getSecrets(client, namespace, "")
+				require.NoError(t, err)
+				configMaps, err := getConfigMaps(client, namespace, "")
+				require.NoError(t, err)
+
+				assert.Empty(t, credentials)
+				assert.Empty(t, configMaps)
+			},
+			finalize: func(t *testing.T, client *k8s.Clientset, namespace string) {
+				ctx, cancel := context.WithTimeout(context.Background(), ctxTimeout)
+				defer cancel()
+
+				err := client.CoreV1().Namespaces().Delete(ctx, namespace, metav1.DeleteOptions{})
+				require.NoError(t, err)
+			},
+		},
+	}
+
+	for tn, tc := range tests {
+		t.Run(tn, func(t *testing.T) {
+			build := getTestBuild(t, func() (common.JobResponse, error) {
+				jobResponse, err := common.GetRemoteBuildResponse(
+					"sleep 5000",
+				)
+				require.NoError(t, err)
+
+				jobResponse.Credentials = []common.Credentials{
+					{
+						Type:     "registry",
+						URL:      "http://example.com",
+						Username: "user",
+						Password: "password",
+					},
+				}
+
+				return jobResponse, nil
+			})
+			build.Runner.Kubernetes.Namespace = tc.namespace
+			buildtest.SetBuildFeatureFlag(build, featureFlagName, featureFlagValue)
+
+			if tc.init != nil {
+				tc.init(t, build, client, tc.namespace)
+			}
+
+			deletedPodNameCh := make(chan string)
+			defer buildtest.OnUserStage(build, func() {
+				ctx, cancel := context.WithTimeout(context.Background(), ctxTimeout)
+				defer cancel()
+				pods, err := client.CoreV1().Pods(tc.namespace).List(
+					ctx,
+					metav1.ListOptions{
+						LabelSelector: labels.Set(build.Runner.Kubernetes.PodLabels).String(),
+					},
+				)
+				require.NoError(t, err)
+				require.NotEmpty(t, pods.Items)
+				pod := pods.Items[0]
+
+				validateResourcesCreated(t, client, featureFlagValue, tc.namespace, pod.Name)
+
+				err = client.
+					CoreV1().
+					Pods(tc.namespace).
+					Delete(ctx, pod.Name, metav1.DeleteOptions{
+						PropagationPolicy: &kubernetes.PropagationPolicy,
+					})
+				require.NoError(t, err)
+
+				deletedPodNameCh <- pod.Name
+			})()
+
+			out, err := buildtest.RunBuildReturningOutput(t, build)
+
+			podName := <-deletedPodNameCh
+
+			if !featureFlagValue {
+				assert.True(t, kubernetes.IsKubernetesPodNotFoundError(err), "expected err NotFound, but got %T", err)
+				assert.Contains(
+					t,
+					out,
+					"ERROR: Job failed (system failure):",
+				)
+			} else {
+				assert.Errorf(t, err, "command terminated with exit code 137")
+			}
+
+			assert.Contains(
+				t,
+				out,
+				fmt.Sprintf("pods %q not found", podName),
+			)
+
+			validateResourcesDeleted(t, client, tc.namespace, podName)
+
+			if tc.finalize != nil {
+				tc.finalize(t, client, tc.namespace)
+			}
+		})
+	}
+}
+
 // TestLogDeletionAttach tests the outcome when the log files are all deleted
 func TestLogDeletionAttach(t *testing.T) {
 	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
@@ -838,6 +1020,109 @@ func testKubernetesPwshFeatureFlag(t *testing.T, featureFlagName string, feature
 	assert.Regexp(t, regexp.MustCompile("PSEdition +Core"), out)
 }
 
+func testKubernetesContainerHookFeatureFlag(t *testing.T, featureFlagName string, featureFlagValue bool) {
+	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
+
+	build := getTestBuild(t, common.GetRemoteSuccessfulBuild)
+	buildtest.SetBuildFeatureFlag(build, featureFlagName, featureFlagValue)
+
+	build.Image.Name = common.TestAlpineImage
+
+	tests := map[string]struct {
+		image           string
+		shell           string
+		lifecycleCfg    common.KubernetesContainerLifecyle
+		steps           common.Steps
+		validateOutputs func(t *testing.T, out string, err error)
+	}{
+		"invalid hook configuration: more than one handler type": {
+			lifecycleCfg: common.KubernetesContainerLifecyle{
+				PreStop: &common.KubernetesLifecycleHandler{
+					Exec: &common.KubernetesLifecycleExecAction{
+						Command: []string{"touch", "/builds/postStart.txt"},
+					},
+					HTTPGet: &common.KubernetesLifecycleHTTPGet{
+						Host: "localhost",
+						Port: 8080,
+					},
+				},
+			},
+			validateOutputs: func(t *testing.T, out string, err error) {
+				require.Error(t, err)
+				assert.Contains(t, out, "ERROR: Job failed (system failure):")
+				assert.Contains(t, out, "may not specify more than 1 handler type")
+			},
+		},
+		"postStart exec hook bash": {
+			steps: common.Steps{
+				common.Step{
+					Name: common.StepNameScript,
+					Script: []string{
+						"ls -l /builds",
+					},
+				},
+			},
+			lifecycleCfg: common.KubernetesContainerLifecyle{
+				PostStart: &common.KubernetesLifecycleHandler{
+					Exec: &common.KubernetesLifecycleExecAction{
+						Command: []string{"touch", "/builds/postStart.txt"},
+					},
+				},
+			},
+			validateOutputs: func(t *testing.T, out string, err error) {
+				require.NoError(t, err)
+				assert.Contains(t, out, "Job succeeded")
+				assert.Contains(t, out, "postStart.txt")
+			},
+		},
+		"postStart exec hook pwsh": {
+			image: common.TestPwshImage,
+			shell: shells.SNPwsh,
+			steps: common.Steps{
+				common.Step{
+					Name: common.StepNameScript,
+					Script: []string{
+						"Get-ChildItem /builds",
+					},
+				},
+			},
+			lifecycleCfg: common.KubernetesContainerLifecyle{
+				PostStart: &common.KubernetesLifecycleHandler{
+					Exec: &common.KubernetesLifecycleExecAction{
+						Command: []string{"touch", "/builds/postStart.txt"},
+					},
+				},
+			},
+			validateOutputs: func(t *testing.T, out string, err error) {
+				require.NoError(t, err)
+				assert.Contains(t, out, "Job succeeded")
+				assert.Contains(t, out, "postStart.txt")
+			},
+		},
+	}
+
+	for tn, tt := range tests {
+		t.Run(tn, func(t *testing.T) {
+			build.Runner.RunnerSettings.Kubernetes.ContainerLifecycle = tt.lifecycleCfg
+
+			if tt.image != "" {
+				build.Image.Name = tt.image
+			}
+
+			if tt.shell != "" {
+				build.Runner.Shell = tt.shell
+			}
+
+			if tt.steps != nil {
+				build.JobResponse.Steps = tt.steps
+			}
+
+			out, err := buildtest.RunBuildReturningOutput(t, build)
+			tt.validateOutputs(t, out, err)
+		})
+	}
+}
+
 func getTestBuild(t *testing.T, getJobResponse func() (common.JobResponse, error)) *common.Build {
 	jobResponse, err := getJobResponse()
 	assert.NoError(t, err)
@@ -871,6 +1156,56 @@ func getTestKubeClusterClient(t *testing.T) *k8s.Clientset {
 	return client
 }
 
+// getSecrets retrieves all the secrets found in the given namespace
+// with at least one ownerReference name matching the name given
+// If ownerName is an empty string, all the secrets resources found are returned
+func getSecrets(client *k8s.Clientset, namespace, ownerName string) ([]v1.Secret, error) {
+	ctx, cancel := context.WithTimeout(context.Background(), 1*time.Minute)
+	defer cancel()
+
+	credList, err := client.CoreV1().Secrets(namespace).List(
+		ctx,
+		metav1.ListOptions{},
+	)
+	if err != nil {
+		return nil, err
+	}
+
+	credentials := []v1.Secret{}
+	for _, cred := range credList.Items {
+		if len(cred.OwnerReferences) == 1 && cred.OwnerReferences[0].Name == ownerName {
+			credentials = append(credentials, cred)
+		}
+	}
+
+	return credentials, nil
+}
+
+// getConfigMaps retrieves all the configMaps found in the given namespace
+// with at least one ownerReference name matching the name given
+// If ownerName is an empty string, all the configMaps resources found are returned
+func getConfigMaps(client *k8s.Clientset, namespace, ownerName string) ([]v1.ConfigMap, error) {
+	ctx, cancel := context.WithTimeout(context.Background(), 1*time.Minute)
+	defer cancel()
+
+	configMapList, err := client.CoreV1().ConfigMaps(namespace).List(
+		ctx,
+		metav1.ListOptions{},
+	)
+	if err != nil {
+		return nil, err
+	}
+
+	configMaps := []v1.ConfigMap{}
+	for _, cfg := range configMapList.Items {
+		if len(cfg.OwnerReferences) == 1 && cfg.OwnerReferences[0].Name == ownerName {
+			configMaps = append(configMaps, cfg)
+		}
+	}
+
+	return configMaps, nil
+}
+
 func runMultiPullPolicyBuild(t *testing.T, build *common.Build) error {
 	build.Runner.Kubernetes.PullPolicy = common.StringOrArray{
 		common.PullPolicyAlways,
@@ -881,8 +1216,7 @@ func runMultiPullPolicyBuild(t *testing.T, build *common.Build) error {
 
 	err := build.Run(&common.Config{}, &common.Trace{Writer: outBuffer})
 	require.Error(t, err)
-	var buildErr *common.BuildError
-	assert.ErrorAs(t, err, &buildErr)
+	assert.ErrorIs(t, err, &common.BuildError{FailureReason: common.ScriptFailure})
 
 	assert.Regexp(
 		t,
@@ -895,3 +1229,171 @@ func runMultiPullPolicyBuild(t *testing.T, build *common.Build) error {
 
 	return err
 }
+
+func TestKubernetesAllowedImages(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
+
+	type testDef struct {
+		AllowedImages []string
+		Image         string
+		VerifyFn      func(*testing.T, error)
+	}
+	tests := map[string]testDef{
+		// allowed image case
+		"allowed image case": {
+			AllowedImages: []string{"alpine"},
+			Image:         "alpine",
+			VerifyFn: func(t *testing.T, err error) {
+				assert.NoError(t, err)
+			},
+		},
+		// disallowed image case
+		"disallowed image case": {
+			AllowedImages: []string{"alpine"},
+			Image:         "ubuntu",
+			VerifyFn: func(t *testing.T, err error) {
+				require.Error(t, err)
+				assert.ErrorIs(t, err, common.ErrDisallowedImage)
+			},
+		},
+	}
+
+	for name, test := range tests {
+		t.Run(name, func(t *testing.T) {
+			successfulBuild, err := common.GetRemoteSuccessfulBuild()
+			assert.NoError(t, err)
+			build := &common.Build{
+				JobResponse: successfulBuild,
+				Runner: &common.RunnerConfig{
+					RunnerSettings: common.RunnerSettings{
+						Executor: "kubernetes",
+						Kubernetes: &common.KubernetesConfig{
+							AllowedImages: test.AllowedImages,
+						},
+					},
+				},
+			}
+			build.Image.Name = test.Image
+
+			err = build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
+
+			test.VerifyFn(t, err)
+		})
+	}
+}
+
+func TestKubernetesAllowedServices(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
+
+	type testDef struct {
+		AllowedServices []string
+		Services        common.Services
+		VerifyFn        func(*testing.T, error)
+	}
+	tests := map[string]testDef{
+		"allowed service case": {
+			AllowedServices: []string{"alpine", "debian"},
+			Services: common.Services{
+				common.Image{Name: "alpine"},
+			},
+			VerifyFn: func(t *testing.T, err error) {
+				assert.NoError(t, err)
+			},
+		},
+		"disallowed service case": {
+			AllowedServices: []string{"alpine", "debian"},
+			Services: common.Services{
+				common.Image{Name: "alpine"},
+				common.Image{Name: "ubuntu"},
+			},
+			VerifyFn: func(t *testing.T, err error) {
+				require.Error(t, err)
+				assert.ErrorIs(t, err, common.ErrDisallowedImage)
+			},
+		},
+	}
+
+	for name, test := range tests {
+		t.Run(name, func(t *testing.T) {
+			successfulBuild, err := common.GetRemoteSuccessfulBuild()
+			assert.NoError(t, err)
+			build := &common.Build{
+				JobResponse: successfulBuild,
+				Runner: &common.RunnerConfig{
+					RunnerSettings: common.RunnerSettings{
+						Executor: "kubernetes",
+						Kubernetes: &common.KubernetesConfig{
+							AllowedServices: test.AllowedServices,
+						},
+					},
+				},
+			}
+			build.Image.Name = "alpine"
+			build.Services = test.Services
+
+			err = build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
+
+			test.VerifyFn(t, err)
+		})
+	}
+}
+
+func TestCleanupProjectGitClone(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
+
+	buildtest.RunBuildWithCleanupGitClone(t, getTestBuild(t, common.GetRemoteSuccessfulBuild))
+}
+
+func TestCleanupProjectGitFetch(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
+
+	untrackedFilename := "untracked"
+
+	build := getTestBuild(t, func() (common.JobResponse, error) {
+		return common.GetRemoteBuildResponse(
+			buildtest.GetNewUntrackedFileIntoSubmodulesCommands(untrackedFilename, "", "")...,
+		)
+	})
+
+	buildtest.RunBuildWithCleanupGitFetch(t, build, untrackedFilename)
+}
+
+func TestCleanupProjectGitSubmoduleNormal(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
+
+	untrackedFile := "untracked"
+	untrackedSubmoduleFile := "untracked_submodule"
+
+	build := getTestBuild(t, func() (common.JobResponse, error) {
+		return common.GetRemoteBuildResponse(
+			buildtest.GetNewUntrackedFileIntoSubmodulesCommands(untrackedFile, untrackedSubmoduleFile, "")...,
+		)
+	})
+
+	buildtest.RunBuildWithCleanupNormalSubmoduleStrategy(t, build, untrackedFile, untrackedSubmoduleFile)
+}
+
+func TestCleanupProjectGitSubmoduleRecursive(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
+
+	untrackedFile := "untracked"
+	untrackedSubmoduleFile := "untracked_submodule"
+	untrackedSubSubmoduleFile := "untracked_submodule_submodule"
+
+	build := getTestBuild(t, func() (common.JobResponse, error) {
+		return common.GetRemoteBuildResponse(
+			buildtest.GetNewUntrackedFileIntoSubmodulesCommands(
+				untrackedFile,
+				untrackedSubmoduleFile,
+				untrackedSubSubmoduleFile)...,
+		)
+	})
+
+	buildtest.RunBuildWithCleanupRecursiveSubmoduleStrategy(
+		t,
+		build,
+		untrackedFile,
+		untrackedSubmoduleFile,
+		untrackedSubSubmoduleFile,
+	)
+}
