@@ -4,6 +4,7 @@ package trace
 
 import (
 	"bytes"
+	"encoding/base64"
 	"io"
 	"math"
 	"math/rand"
@@ -86,6 +87,43 @@ func TestVariablesMaskingBoundary(t *testing.T) {
 			values:   []string{"zzz"},
 			expected: "head slice\n" + strings.Repeat(".", 4095) + "tail slice",
 		},
+
+		// large secrets / flushing on certain boundaries that exceed the internal text/transform
+		// buffer results in short writes.
+		// https://gitlab.com/gitlab-org/gitlab-runner/-/issues/27964
+		{
+			input:    "large secret, but no match: " + strings.Repeat("_", 3000) + "|" + strings.Repeat("+", 3000),
+			values:   []string{strings.Repeat("_", 8000)},
+			expected: "large secret, but no match: " + strings.Repeat("_", 3000) + strings.Repeat("+", 3000),
+		},
+		//nolint:lll
+		{
+			input:    "large secret, full sized/full match: " + strings.Repeat("_", maxPhraseSize/2) + "|" + strings.Repeat("_", maxPhraseSize-(maxPhraseSize/2)),
+			values:   []string{strings.Repeat("_", maxPhraseSize)},
+			expected: "large secret, full sized/full match: [MASKED]",
+		},
+		//nolint:lll
+		{
+			input:    "large secret, over sized/partial match/tailing reveal: " + strings.Repeat("_", maxPhraseSize/2) + "|" + strings.Repeat("_", maxPhraseSize-(maxPhraseSize/2)) + "endsuffix",
+			values:   []string{strings.Repeat("_", maxPhraseSize) + "endsuffix"},
+			expected: "large secret, over sized/partial match/tailing reveal: [MASKED]endsuffix",
+		},
+		{
+			input:    "large secret, 2x mask: " + strings.Repeat("_", 3000) + "|" + strings.Repeat("_", 3000),
+			values:   []string{strings.Repeat("_", 6000)},
+			expected: "large secret, 2x mask: [MASKED]" + strings.Repeat("_", 6000-maxPhraseSize),
+		},
+		{
+			input:    "large secret mask in single write: " + strings.Repeat("_", 6000),
+			values:   []string{strings.Repeat("_", 6000)},
+			expected: "large secret mask in single write: [MASKED]",
+		},
+		//nolint:lll
+		{
+			input:    "undersized partial matches should have no masks: __nomatch __|small __" + strings.Repeat("0", 3000) + "|" + strings.Repeat("0", 3000),
+			values:   []string{"__x"},
+			expected: "undersized partial matches should have no masks: __nomatch __small __" + strings.Repeat("0", 3000) + strings.Repeat("0", 3000),
+		},
 	}
 
 	for _, tc := range tests {
@@ -113,6 +151,59 @@ func TestVariablesMaskingBoundary(t *testing.T) {
 	}
 }
 
+func TestPhraseFind(t *testing.T) {
+	tests := []struct {
+		input  string
+		phrase string
+		match  matchType
+		index  int
+	}{
+		{
+			input: "this text [secret] contains the secret",
+			match: fullMatch,
+			index: 10,
+		},
+		{
+			input: "this text has no secret",
+			match: noPossibleMatch,
+			index: 23,
+		},
+		{
+			input: "within this text [secret] there's two [secret]s",
+			match: fullMatch,
+			index: 17,
+		},
+		{
+			input: "within this text there's a partial [secre",
+			match: partialMatch,
+			index: 35,
+		},
+		{
+			input: "within this text there's almost a [secret followed by a full [secret]",
+			match: fullMatch,
+			index: 61,
+		},
+		{
+			input: "within this text there's almost a [secret followed by a partial [se",
+			match: partialMatch,
+			index: 64,
+		},
+		{
+			input: "within this text there's a [[secret]",
+			match: fullMatch,
+			index: 28,
+		},
+	}
+
+	for _, tc := range tests {
+		t.Run(tc.input, func(t *testing.T) {
+			match, index := find([]byte(tc.input), []byte("[secret]"))
+			assert.Equal(t, tc.match, match)
+			assert.Equal(t, tc.index, index)
+		})
+	}
+}
+
 func TestMaskNonEOFSafeBoundary(t *testing.T) {
 	// The truncated output from unflushed results depends on the max token
 	// size we're trying to find.
@@ -124,15 +215,15 @@ func TestMaskNonEOFSafeBoundary(t *testing.T) {
 	}{
 		{
 			input:    "cannot safely flush: secret secre",
-			expected: "cannot safely flush: [MASKED]",
+			expected: "cannot safely flush: [MASKED] ",
 		},
 		{
 			input:    "cannot safely flush: secret secre!",
-			expected: "cannot safely flush: [MASKED] ",
+			expected: "cannot safely flush: [MASKED] secre!",
 		},
 		{
 			input:    "cannot safely flush: secret secre\t",
-			expected: "cannot safely flush: [MASKED] ",
+			expected: "cannot safely flush: [MASKED] secre\t",
 		},
 		{
 			input:    "can safely flush: secret secre\r",
@@ -148,8 +239,8 @@ func TestMaskNonEOFSafeBoundary(t *testing.T) {
 		},
 		//nolint:lll
 		{
-			input:    "can safely flush: \n: but doesn't use the last safe token if the input is much greater than the token being indexed",
-			expected: "can safely flush: \n: but doesn't use the last safe token if the input is much greater than the token being i",
+			input:    "can safely flush: \n: but doesn't use the last safe token if the input is much greater than the token being indexed sec",
+			expected: "can safely flush: \n: but doesn't use the last safe token if the input is much greater than the token being indexed ",
 		},
 		{
 			input:    "can safely flush: \n: and always uses the last safe token even on long inputs\n",
@@ -226,3 +317,52 @@ func TestRandomCopyReadback(t *testing.T) {
 
 	assert.Equal(t, []byte(expected), content)
 }
+
+func TestMaskLargePartialMasks(t *testing.T) {
+	count := 20
+	chunkSize := maxPhraseSize / 2
+
+	generateRandom := func(size int) string {
+		buf := make([]byte, size)
+		_, err := rand.Read(buf)
+		require.NoError(t, err)
+
+		return base64.StdEncoding.EncodeToString(buf)
+	}
+
+	var secrets []string
+	for i := 1; i <= count; i++ {
+		secrets = append(secrets, generateRandom(i*chunkSize))
+	}
+
+	var unmasked []string
+	for i := 1; i <= count; i++ {
+		unmasked = append(unmasked, generateRandom(i*chunkSize))
+	}
+
+	buffer, err := New()
+	require.NoError(t, err)
+	defer buffer.Close()
+
+	buffer.SetMasked(secrets)
+
+	for i := range secrets {
+		n, err := buffer.Write([]byte(secrets[i]))
+		require.NoError(t, err)
+		assert.Equal(t, len(secrets[i]), n)
+
+		n, err = buffer.Write([]byte(unmasked[i]))
+		require.NoError(t, err)
+		assert.Equal(t, len(unmasked[i]), n)
+	}
+
+	buffer.Finish()
+
+	content, err := buffer.Bytes(0, math.MaxInt64)
+	require.NoError(t, err)
+
+	for i := range secrets {
+		assert.NotContains(t, string(content), secrets[i], "contains secret %d", i)
+		assert.Contains(t, string(content), unmasked[i], "doesn't contain safe unmasked %d", i)
+	}
+}
