@@ -9,21 +9,17 @@ import (
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 )
 
-// BashTrapShellScript is used to wrap a shell script in a trap that makes sure the script always exits with exit code of 0
-// this can be useful in container environments where exiting with an exit code different from 0 would kill the container.
-// At the same time it writes to a file the actual exit code of the script as well as the filename of the script as json.
+// BashTrapShellScript is used to wrap a shell script in a trap that makes sure the script always exits
+// with exit code of 0 this can be useful in container environments where exiting with an exit code different from 0
+// would kill the container.
+// At the same time it writes to a file the actual exit code of the script as well as the filename
+// of the script as json.
 const bashTrapShellScript = `runner_script_trap() {
 	exit_code=$?
-	log_file=%s
 	out_json="{\"command_exit_code\": $exit_code, \"script\": \"$0\"}"
 
-	# Make sure the command status will always be printed on a new line 
-	if [[ $(tail -c1 $log_file | wc -l) -gt 0 ]]; then
-		printf "$out_json\n" >> $log_file
-	else 
-		printf "\n$out_json\n" >> $log_file
-	fi
-	
+	echo ""
+	echo "$out_json"
 	exit 0
 }
 
@@ -51,7 +47,7 @@ func (b *BashTrapShellWriter) Finish(trace bool) string {
 }
 
 func (b *BashTrapShellWriter) writeTrap(w io.Writer) {
-	_, _ = fmt.Fprintf(w, bashTrapShellScript, b.logFile)
+	_, _ = fmt.Fprint(w, bashTrapShellScript)
 }
 
 type BashTrapShell struct {
